#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <assert.h>
#include <test-api-common.h>

#define BIG_STRSIZE 1024 * 1024
char *helloWorld = NULL;

void
testInit(tlssep_context_t *context, tlssep_status_t expected)
{
	if (NULL == helloWorld) {
		helloWorld = malloc(BIG_STRSIZE + 1);
		assert(NULL != helloWorld);

		int i;
		for (i = 0; i < BIG_STRSIZE; i++) {
			helloWorld[i] = 'A';
		}
		helloWorld[BIG_STRSIZE] = 0x00;
	}

        tlssep_status_t status = tlssep_init(context);
	if (expected != status) {
                fprintf(stderr, "Init test failed: got %s, expected %s\n", tlssep_strerror(status), tlssep_strerror(expected));
                exit(EXIT_FAILURE);
        }
}

void
testWrite(tlssep_desc_t *desc, tlssep_status_t expected)
{
        int num_written;
        tlssep_status_t status = tlssep_write(desc, helloWorld, strlen(helloWorld) + 1, &num_written);
	if (expected != status) {
                fprintf(stderr, "Write test failed: got %s, expected %s\n", tlssep_strerror(status), tlssep_strerror(expected));
                exit(EXIT_FAILURE);
        }
	if (TLSSEP_STATUS_OK == expected) {
		assert(num_written == strlen(helloWorld) + 1);
	}
}

void
testPeek(tlssep_desc_t *desc, tlssep_status_t expected)
{
	char buf[BUFSIZ + 1];
	int num_read = 0;
	tlssep_status_t status = tlssep_peek(desc, buf, BUFSIZ, &num_read);
	if (expected != status) {
		fprintf(stderr, "Peek test failed: got %s, expected %s\n", tlssep_strerror(status), tlssep_strerror(expected));
		exit(EXIT_FAILURE);
	}
	buf[BUFSIZ] = 0x00;
        if (TLSSEP_STATUS_OK == expected) {
		assert(strlen(buf) == num_read);
		assert(0 == strncmp(buf, helloWorld, BUFSIZ));
        }
}

void
testRead(tlssep_desc_t *desc, tlssep_status_t expected)
{
        char buf[BIG_STRSIZE + 1];
        int num_read = 0, total = 0;
	do {
		tlssep_status_t status = tlssep_read(desc, buf + total, sizeof(buf) - total, &num_read);
		if (expected != status) {
			fprintf(stderr, "Read test failed: got %s, expected %s\n", tlssep_strerror(status), tlssep_strerror(expected));
			exit(EXIT_FAILURE);
		}
		total += num_read;
	} while (sizeof(buf) != total);
	if (TLSSEP_STATUS_OK == expected) {
		assert(strlen(helloWorld) + 1 == total);
		assert(strlen(buf) + 1 == total);
		assert(0 == strcmp(buf, helloWorld));
	}
}

void
testReadRemoteClose(tlssep_desc_t *desc, tlssep_status_t expected)
{
	/* This read should return length 0 because we hope
	 * the peer has closed the connection.
	 */
        char buf[BIG_STRSIZE + 1];
        int num_read = 0;
        tlssep_status_t status = tlssep_read(desc, buf, sizeof(buf), &num_read);
	if (expected != status) {
                fprintf(stderr, "Read test failed: got %s, expected %s\n", tlssep_strerror(status), tlssep_strerror(expected));
                exit(EXIT_FAILURE);
        }
	if (TLSSEP_STATUS_OK == expected) {
		assert(0 == num_read);
	}
}

void
testClose(tlssep_desc_t *desc, tlssep_status_t expected)
{
	tlssep_status_t status = tlssep_close(desc);
	if (expected != status) {
                fprintf(stderr, "Close test failed: got %s, expected %s\n", tlssep_strerror(status), tlssep_strerror(expected));
                exit(EXIT_FAILURE);
        }
}

void
testTerminate(tlssep_context_t *context)
{
	tlssep_status_t status = tlssep_terminate(context);
	assert(TLSSEP_STATUS_OK == status);
}
